'' =================================================================================================
''
''   File....... jm_plx_badge_test_2015-08-19.spin
''   Purpose.... Parallax Electronic Conference Badge Test
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 19 AUG 2015
''               -- enhanced IR checking (eliminate false detection)
''               -- added hacker pad labels as embedded comments
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us

  IR_BAUD  =   2_400  { max supported using IR connection }
  IR_FREQ  =  36_000  { matches receiver on DC22 badge }

  T_BAUD   = 115_200  { for terminal io } 
  

con { io pins }

  RX1      = 31                                                  ' programming / terminal
  TX1      = 30

  SDA      = 29                                                  ' eeprom / i2c
  SCL      = 28

  BTN_0    = 27                                                  ' touch buttons (right)
  BTN_1    = 26
  BTN_2    = 25

  IR_OUT   = 24                                                  ' IR coms
  IR_IN    = 23
  
  OLED_DAT = 22                                                  ' OLED connections 
  OLED_CLK = 21
  OLED_DC  = 20  
  OLED_RST = 19  
  OLED_CS  = 18

  BTN_5    = 17                                                  ' touch buttons (left)
  BTN_4    = 16
  BTN_3    = 15  

  TV_DAC2  = 14                                                  ' composite video (J503)  
  TV_DAC1  = 13
  TV_DAC0  = 12
  
  AUD_R    = 10  { AUDR }                                        ' audio (J503)
  AUD_L    =  9  { AUDL }

  BLU_CP2  =  8  { LEDc }                                        ' blue led charlieplex pins 
  BLU_CP1  =  7  { LEDb }
  BLU_CP0  =  6  { LEDa }

  BTN_OS   =  5                                                  ' open source logo button
  
  ACC_INT  =  4  { XYZi }                                        ' accelerometer interrupt in
  
  RGB_CP2  =  3  { RGBc }                                        ' rgb led charlieplex pins 
  RGB_CP1  =  2  { RGBb } 
  RGB_CP0  =  1  { RGBa } 

  BATT_MON =  0  { RCTv }                                        ' battery monitor(go/no-go)


con { io configuration }

  IS_OFF    =  0                                                 ' all bits off   
  IS_ON     = -1                                                 ' all bits on    

  IS_LOW    =  0                                                
  IS_HIGH   = -1                                                

  IS_INPUT  =  0
  IS_OUTPUT = -1
  

con { buttons masks }
  
  PB0_MASK  = %0_000_001                                         ' upper right
  PB1_MASK  = %0_000_010                                         ' middle right
  PB2_MASK  = %0_000_100                                         ' lower right
  PB3_MASK  = %0_001_000                                         ' lower left
  PB4_MASK  = %0_010_000                                         ' middle left
  PB5_MASK  = %0_100_000                                         ' upper left
  PB6_MASK  = %1_000_000                                         ' open source logo

  OFF_MASK  = PB5_MASK | PB0_MASK
  

dat { touch pads configuration }

  TPCount       byte    7
  TPPins        byte    BTN_OS, BTN_5, BTN_4, BTN_3, BTN_2, BTN_1, BTN_0
  TPDischarge   byte    15
  

con { led control }

  #0, BLUE_0, BLUE_1, BLUE_2, BLUE_3, BLUE_4, BLUE_5             ' charlieplex control for blue leds
  #0, RGB_B1, RGB_G1, RGB_R1, RGB_B2, RGB_G2, RGB_R2             ' charlieplex control for rgb leds


obj

' main                                                           ' * master Spin cog
  io   : "jm_io"                                                 '   io control
  pads : "jm_touchpads"                                          '   touchpad input
  time : "jm_time"                                               '   timing and delays
  ee   : "jm_24xx512"                                            '   eeprom access 
  acc  : "jm_mma7660fc"                                          '   MMA7660 accelerometer
  leds : "jm_ebadge_leds"                                        ' * charlieplexing driver for LEDs
' oled : "jm_ssd1306_spi_ez"                                     '   OLED display driver
  oled : "oled_asmfast"                                          ' * OLED driver (TEMPORARY)
' ir   : "jm_ir_hdserial"                                        ' * half-duplex serial via IR
  term : "jm_fullduplexserial"                                   ' * full-duplex serial via terminal

' * Uses cog when loaded


var

  long  cpcog                                                    ' charlieplexing cog


dat { badge messages }

' Important:
' -- Ensure both lines of message are 8 characters long
                            
  Msg0_0        byte    "PARALLAX", 0 
  Msg0_1        byte    " eBADGE ", 0

  Msg1_0        byte    "LEDS    ", 0 
  Msg1_1        byte    "        ", 0

  Msg2_0        byte    "BUTTONS ", 0 
  Msg2_1        byte    "%0000000", 0  

  Msg3_0        byte    "ACCEL   ", 0 
  Msg3_1        byte    "- ?     ", 0

  Msg4_0        byte    "EEPROM  ", 0 
  Msg4_1        byte    "- ??K   ", 0

  Msg5_0        byte    "IR      ", 0 
  Msg5_1        byte    "- ?     ", 0

 

pub main

  setup

  ' show open source logo

  bytemove(oled.getBuffer, @OS_Logo, 1024)        
  oled.updateDisplay

  ' just for fun, let's wiggle it back and forth

  oled.startScrollRight($00, $0F)                                ' scroll all 16 rows  
  time.pause(750)
  oled.startScrollLeft($00, $0F)                                ' scroll all 16 rows  
  time.pause(1500)
  oled.startScrollRight($00, $0F)                                ' scroll all 16 rows  
  time.pause(750)  
  oled.stopScroll
  time.pause(250)                                                             


  show_screen(@Msg0_0)
  time.pause(2000)
  

  ' Tip: Preset PST to correct com port and 115_200 baud
  '      -- press [F10], [F12] (to open PST), then [Enter] (in PST)

  term.rxflush                                                  ' clear buffer
  term.tx(term#CLS)
  term.str(string("Parallax eBadge Test"))
  repeat 2
    term.tx(term#CR)


  ' test regime

  test_leds
  test_buttons
  test_acc                                                       ' test accelerometer
  test_eeprom                                                    ' verify 64K
  test_ir                                                        ' check ir out and in


  show_screen(@Msg0_0) 
  
  repeat
    waitcnt(0)

  

pub test_leds

  term.str(string("1) Verifying LEDs", 13))
  show_screen(@Msg1_0)

  repeat 3
    leds.set_blue(%100001)                                       ' p17   p27
    leds.set_rgbx(leds#RED, leds#RED)
    time.pause(300)
    leds.set_blue(%010010)                                       ' p16   p26
    leds.set_rgbx(leds#GREEN, leds#GREEN)
    time.pause(300)
    leds.set_blue(%001100)                                       ' p15   p25 
    leds.set_rgbx(leds#BLUE, leds#BLUE)                          
    time.pause(300)
    leds.set_all(0)
    time.pause(100) 
          

pub test_buttons | bcheck, b, p_bits, idx

  term.str(string("2) Verifying Buttons", 13))
  show_screen(@Msg2_0) 

  bcheck := %0000000                                             ' verified buttons

  repeat
    repeat
      b := pads.read_pads
    until (b <> %0000000)
    bcheck |= b
  
    ' move bmask into display

    p_bits := @Msg2_1 + 1                                        ' skip past %
    repeat idx from 6 to 0
      if (bcheck & (1 << idx))
        byte[p_bits++] := "1"
      else
        byte[p_bits++] := "0"
    show_screen(@Msg2_0)

    if (bcheck == %1111111)
      time.pause(100)
      quit


pub test_acc | xg, yg, zg, tilt, t

  term.str(string("3) Verifying Accelerometer", 13))
  show_screen(@Msg3_0)

  ifnot (acc.present)                                            ' check for MMA7660
    oled.write2x8String(string("- Fail"), 6, 1)
    oled.updateDisplay         
    repeat
      waitcnt(0)
  else
    oled.write2x8String(string("- Okay"), 6, 1)
    oled.updateDisplay
    time.pause(1000)
    oled.clearDisplay 

  t := cnt
  repeat 20                                                      ' show g-force for 2s
    acc.read_all(@xg)                                            ' read all axes + tilt       
    oled.write4x16String(string("X: "), 3, 2, 4)  
    show_gforce(xg, 2, 7)
    oled.write4x16String(string("Y: "), 3, 3, 4) 
    show_gforce(yg, 3, 7) 
    oled.write4x16String(string("Z: "), 3, 4, 4)   
    show_gforce(zg, 4, 7)                          
    oled.updateDisplay
    waitcnt(t += (100 * MS_001))


pub test_eeprom | rd0, rd32k, wr32k, check, size

  term.str(string("4) Checking EEPROM Size", 13))
  show_screen(@Msg4_0)

  rd0   := ee.rd_byte($0000)                                     ' read addres 0
  rd32k := ee.rd_byte($8000)                                     ' read addres 32k (upper half of 64k)

  if (rd0 <> rd32k)                                              ' if not equal
    bytemove(@Msg4_1, string("- 64K"), 5)                        '  assume 64k
    size := 64

  else
    wr32k := !rd32k & $FF                                        ' create write test
    ee.wr_byte($8000, wr32k)                                     ' write to 1st byte of upper half
    check := ee.rd_byte($0000)                                   ' if addr 0 updated, ee size is 32k  
    if (rd0 == wr32k)
      bytemove(@Msg4_1, string("- 32K"), 5)
      size := 32
      ee.wr_byte($0000, rd0)                                     ' repair after test 
       
  oled.AutoUpdateOff
  oled.write2x8String(@Msg4_1, 8, 1)
  oled.updateDisplay
  term.str(string("   - "))
  term.dec(size)
  term.tx("K")
  term.tx(term#CR)
  time.pause(1000)
  oled.clearDisplay


pub test_ir | check

  term.str(string("5) Testing IR", 13))
  show_screen(@Msg5_0)

  if (ina[IR_IN] == 0)                                           ' added 19 AUG 2015                           
    oled.write2x8String(string("- Fail"), 6, 1)
    oled.updateDisplay  
    time.pause(2000)    
    return

  ' sensor seems okay, now turn on IR source  

  io.freq_out("A", IR_OUT, IR_FREQ)                              ' activate ir modulation

  repeat 20
    if (ina[IR_IN] == 0)
      leds.set_rgbx(leds#RED, leds#RED)
      oled.write2x8String(string("- Yes"), 5, 1) 
    else
      leds.set_all(0)
      oled.write2x8String(string("- No "), 5, 1)

    oled.updateDisplay
    time.pause(100)

  io.freq_out("A", IR_OUT, 0)                                    ' turn off IR source
  leds.set_rgbx(0, 0)                                            ' clear rgb LEDs
 

pub show_gforce(g, row, col) | div

'' Show g-force value as n.nn 

  if (g < 0)
    oled.write5x7Char("-", row, col++)   
    g := ||g
  elseif (g == 0)
    oled.write5x7Char(" ", row, col++)   
  else
    oled.write5x7Char("+", row, col++)

  oled.write5x7Char(("0" + (g / 100)), row, col++)               ' print 100s   
  oled.write5x7Char(".", row, col++)                             ' decimal point                                               
  g //= 100                                                      ' remove 100s
  oled.write5x7Char(("0" + (g  / 10)), row, col++)               ' print 10s  
  oled.write5x7Char(("0" + (g // 10)), row, col++)               ' print 1s   


pub show_screen(p_scr)

'' Displays two 8-line messages on OLED
'' -- uses Parallax font
'' -- both lines must be 8 characters wide

  oled.AutoUpdateOff

  oled.write2x8String(p_scr+0, 8, 0) 
  oled.write2x8String(p_scr+9, 8, 1)

  oled.updateDisplay


pub setup                                

'' Setup IO and objects for application

  io.start(0, 0)                                                 ' clear all pins (this cog)

  pads.start(TPCount, @TPPins, TPDischarge)                      ' setup touchpad interface

  time.start                                                     ' setup timing/delays for clock speed

  ee.start(%000)                                                 ' connect to boot eeprom

  acc.start(SCL, SDA)                                            ' connect to accelerometer

  cpcog := leds.start                                            ' start charlieplexing driver

  oled.init(OLED_CS, OLED_DC, OLED_DAT, OLED_CLK, OLED_RST, {    ' start buffered oled driver
}           oled#SSD1306_SWITCHCAPVCC, oled#TYPE_128X64)
  

' ir.start(IR_IN, IR_OUT, IR_BAUD, IR_FREQ)                      ' start IR serial  
  
  term.start(RX1, TX1, %0000, T_BAUD)                            ' start serial for terminal


con

  { --------------------------- }
  {  E B A D G E   B A S I C S  }
  { --------------------------- }


pub set_1_blue(idx)                                              ' LOCKED 30 JUN 2015  

'' Set one of the six blue LEDs
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?
    if ((idx => BLUE_0) and (idx =< BLUE_5))                     ' valid element?
      leds.set_blue(1 << idx)                                    ' set via mask
    else
      leds.set_blue(%000000)                                     ' clear all 
    return 

  ' if no driver loaded, use direct control

  case idx
    BLUE_0:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_1:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L    
      dira[BLU_CP2..BLU_CP0] := %101

    BLUE_2:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H 
      dira[BLU_CP2..BLU_CP0] := %101      

    BLUE_3:
      outa[BLU_CP2..BLU_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z  
      dira[BLU_CP2..BLU_CP0] := %110

    BLUE_4:
      outa[BLU_CP2..BLU_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[BLU_CP2..BLU_CP0] := %011
      
    BLUE_5:
      outa[BLU_CP2..BLU_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H
      dira[BLU_CP2..BLU_CP0] := %011  

    other:
      outa[BLU_CP2..BLU_CP0] := %000                             ' disable all
      dira[BLU_CP2..BLU_CP0] := %000         


pub set_1_rgb(idx)                                               ' LOCKED 30 JUN 2015  

'' Set one of the LEDs within the RGB modules
'' -- connections designed for Charlieplexing
''    * make one high, one low, the other high-z (input)
'' -- for multiple LEDs/colors use Charlieplex driver methods

  if (cpcog)                                                     ' Charlieplex driver loaded?      
    if ((idx => RGB_B1) and (idx =< RGB_R2))                     ' valid element?                  
      leds.set_rgb(1 << idx)                                     ' set via mask                    
    else                                                                                           
      leds.set_rgb(%000000)                                      ' clear all                       
    return

  ' if no driver loaded, use direct control

  case idx
    RGB_B1:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = Z, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = Z, CP1 = H, CP0 = L 
      dira[RGB_CP2..RGB_CP0] := %011
  
    RGB_R1:
      outa[RGB_CP2..RGB_CP0] := %010                             ' CP2 = L, CP1 = H, CP0 = Z     
      dira[RGB_CP2..RGB_CP0] := %110

    RGB_B2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = L, CP1 = Z, CP0 = H  
      dira[RGB_CP2..RGB_CP0] := %101

    RGB_G2:
      outa[RGB_CP2..RGB_CP0] := %001                             ' CP2 = Z, CP1 = L, CP0 = H 
      dira[RGB_CP2..RGB_CP0] := %011      

    RGB_R2:
      outa[RGB_CP2..RGB_CP0] := %100                             ' CP2 = H, CP1 = L, CP0 = Z
      dira[RGB_CP2..RGB_CP0] := %110  
  
    other:
      outa[RGB_CP2..RGB_CP0] := %000                             ' disable all
      dira[RGB_CP2..RGB_CP0] := %000 


dat { Propeller beanie logo }

  OS_Logo       byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $F0, $FC, $FC, $FC, $FC
                byte    $FC, $FC, $FC, $FC, $FC, $F0, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $C0, $E0, $F0, $F8, $FC, $FE, $FE, $FC, $FC, $F8
                byte    $F0, $E0, $E0, $C0, $C0, $E0, $E0, $E0, $F0, $F0, $FC, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $F0, $E0, $E0, $E0, $C0, $C0, $E0, $E0
                byte    $F0, $F8, $FC, $FC, $FE, $FE, $FC, $F8, $F0, $E0, $C0, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $01, $03, $0F, $1F, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $1F, $0F, $03, $01, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80
                byte    $80, $80, $80, $C0, $C0, $C0, $C0, $C0, $F0, $FC, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $3F, $1F, $0F, $07, $07, $07, $03
                byte    $03, $03, $07, $07, $07, $0F, $1F, $3F, $7F, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $C0, $C0, $C0, $C0, $C0, $80, $80
                byte    $80, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $81, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $81, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $01
                byte    $01, $01, $01, $01, $01, $03, $03, $03, $0F, $7F, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FE, $FC, $F8, $F0, $20, $00, $00, $00
                byte    $00, $00, $00, $00, $20, $F0, $F8, $FC, $FE, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $0F, $03, $03, $03, $01, $01, $01, $01
                byte    $01, $01, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $80, $E0, $F0, $FC, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $7F, $1F, $07, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $07, $1F, $7F, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FC, $F0, $E0, $80, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $01, $03, $07, $0F, $1F, $3F, $7F, $3F, $1F, $0F
                byte    $0F, $07, $03, $03, $07, $07, $0F, $03, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $03, $0F, $07, $07, $03, $03, $07
                byte    $0F, $0F, $1F, $3F, $7F, $3F, $1F, $0F, $07, $03, $01, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                         

  Beanie        byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0
                byte    $C0, $C0, $C0, $C0, $C0, $C0, $C0, $C0, $80, $80, $80, $80, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $80
                byte    $80, $00, $00, $00, $80, $80, $80, $80, $C0, $C0, $C0, $C0, $C0, $E0, $E0, $E0
                byte    $E0, $E0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0, $F0
                byte    $E0, $E0, $80, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $0F, $1F, $3F
                byte    $3F, $7F, $7F, $7F, $7F, $7F, $7F, $7F, $7F, $7F, $7F, $7F, $3F, $3F, $3F, $3F
                byte    $3F, $3F, $1F, $1F, $1E, $1E, $1E, $0E, $0E, $0E, $0E, $06, $06, $06, $F7, $FF
                byte    $FF, $F7, $03, $03, $03, $03, $03, $03, $03, $03, $03, $03, $03, $03, $03, $07
                byte    $07, $07, $07, $07, $07, $0F, $0F, $0F, $0F, $0F, $1F, $1F, $1F, $1F, $1F, $1F
                byte    $0F, $0F, $07, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $80, $C0, $C0, $E0, $E0, $60, $70, $30, $30, $18, $18, $C8, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $C8, $18, $18, $30, $30, $70, $60, $E0, $E0, $C0, $C0, $80, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $80, $C0, $E0, $F0, $F8, $FC, $FE, $7F
                byte    $3F, $0F, $07, $03, $01, $00, $00, $00, $00, $C0, $FC, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FC, $C0, $00, $00, $00, $00, $01, $03, $07, $0F, $3F
                byte    $7F, $FE, $FC, $F8, $F0, $E0, $C0, $80, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $80, $E0, $F8, $FC, $FF, $FF, $FF, $FF, $FF, $3F, $07, $01, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $F8, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $F8, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $01, $07, $3F, $FF, $FF, $FF, $FF, $FF, $FC, $F8, $E0, $80, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $C0, $FC, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $BF, $81, $80, $80, $80, $C0
                byte    $C0, $C0, $C0, $C0, $C0, $C0, $C0, $F0, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF
                byte    $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $F0, $C0, $C0, $C0, $C0, $C0, $C0, $C0
                byte    $C0, $80, $80, $80, $81, $BF, $FF, $FF, $FF, $FF, $FF, $FF, $FF, $FC, $C0, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $78, $FF, $FF, $FF, $FF, $FF, $FF, $CF, $CF, $CF, $CF, $CF, $C7, $87, $87, $87
                byte    $87, $87, $87, $87, $87, $87, $87, $07, $03, $03, $03, $03, $03, $03, $03, $03
                byte    $03, $03, $03, $03, $03, $03, $03, $03, $07, $87, $87, $87, $87, $87, $87, $87
                byte    $87, $87, $87, $C7, $CF, $CF, $CF, $CF, $CF, $FF, $FF, $FF, $FF, $FF, $FF, $78
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $01, $01, $03, $03, $03, $03, $03, $07, $07, $07, $07, $07, $07, $07
                byte    $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F
                byte    $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F, $0F
                byte    $07, $07, $07, $07, $07, $07, $07, $03, $03, $03, $03, $03, $01, $01, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                byte    $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00
                         

dat { license }

{{

  Copyright (C) 2015 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}